/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"

GtkWidget *images_combo;
GtkWidget *update_btn;

static void
populate_store (gpointer k, gpointer v, gpointer data)
{
    GtkListStore *store;
    GtkTreeIter iter;

    store = (GtkListStore *) data;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter,
                        0, (gchar *) k, 1, (gchar *) v,
                        -1);
}

static void
on_dialog_response (GtkDialog *dialog, gint arg1, gpointer data)
{
    if (arg1 != GTK_RESPONSE_OK)
        gtk_main_quit ();

    gboolean do_update;
    const gchar *login;
    const gchar *passwd;
    const gchar *image_id;

    do_update = gtk_toggle_button_get_active (
        GTK_TOGGLE_BUTTON (update_btn));
    image_id = get_selected_image (GTK_COMBO_BOX (images_combo));

    if (do_update) {
        if (!xfce_confirm (
                _("Your system will now reboot and perform the upgrade."
                  "You can still cancel this upgrade.\n\n"
                  "Do you really want to reboot and perform the upgrade NOW?"),
                  GTK_STOCK_YES, NULL)) {
           return;
        }

        login = (const gchar *) g_object_get_data (G_OBJECT (dialog), "login");
        passwd = (const gchar *) g_object_get_data (G_OBJECT (dialog), "passwd");
        token *t;
        /* request a token on the server */
        t = get_token (login, passwd, image_id);
        if (t->error)
        {
            xfce_err (_("The server returned the error:\n%s"), t->error);
            _exit (0);
        }

        if (login && passwd)
        {
            save_config ("login", login, "passwd", passwd,
                         "image", image_id, "token", t->token, NULL);
        }
        else
            save_config ("image", image_id, "token", t->token, NULL);

        /* prepare the system */
        if (!get_prep_is_done() && !set_prep (TRUE))
        {
            warn_setup_error ();
            gtk_main_quit ();
        }

        system ("reboot");

    } else {
        if (get_prep_is_done() && !set_prep (FALSE))
        {
            warn_setup_error ();
            gtk_main_quit ();
        }
        save_config (NULL);
    }

    gtk_main_quit ();
}

void
update_dialog (status *s, const gchar *login, const gchar *passwd)
{
    GtkWidget *dialog;
    GtkWidget *label, *hbox;
    GtkListStore *images_store;
    GtkCellRenderer *renderer;
    const gchar *whoyouare;

    dialog = xfce_titled_dialog_new_with_buttons (
            "Account informations",
            NULL, GTK_DIALOG_MODAL,
            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
            GTK_STOCK_OK, GTK_RESPONSE_OK,
            NULL);
    if (login && passwd) {
        g_object_set_data (G_OBJECT (dialog), "login", (gpointer) login);
        g_object_set_data (G_OBJECT (dialog), "passwd", (gpointer) passwd);
    }

    if (login)
        whoyouare = g_strdup_printf ("You are logged in as %s", login);
    else
        whoyouare = g_strdup ("You are not logged in (anonymous update)");

    label = gtk_label_new (whoyouare);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
                        FALSE, FALSE, 0);
    gtk_widget_show (label);

    if (s->count >= 0)
    {
        gchar *count_txt = NULL;

        count_txt = g_strdup_printf ("Remaining updates: %d", s->count);
        label = gtk_label_new (count_txt);
        gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
        g_free (count_txt);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
                            FALSE, FALSE, 0);
        gtk_widget_show (label);
    }

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
                        FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("Available system images: ");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
    gtk_widget_show (label);

    images_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
    g_hash_table_foreach (s->images, populate_store, images_store);
    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (images_store),
                                     1, str_cmp_func, NULL, NULL);
    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (images_store),
                                          1, GTK_SORT_ASCENDING);

    images_combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (images_store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (images_combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (images_combo), renderer,
                                   "text", 1);
    gtk_combo_box_set_active (GTK_COMBO_BOX (images_combo), 0);
    gtk_box_pack_start (GTK_BOX (hbox), images_combo, FALSE, FALSE, 2);
    gtk_widget_show (images_combo);

    update_btn = gtk_check_button_new_with_label (
        "Prepare the system for an upgrade using this image at next boot");
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), update_btn,
                        FALSE, FALSE, 2);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (update_btn),
                                  get_prep_is_done ());
    gtk_widget_show (update_btn);

    g_signal_connect (G_OBJECT (dialog), "response",
                      G_CALLBACK (on_dialog_response), NULL);
    gtk_widget_show (dialog);
}
